@@ -1,3 +1,5 @@
+import sys ; sys.path.insert(0, "/home/gonzo/ReposForks/yfinance-ValueRaider.integrate")
+
 import unittest
 from pprint import pprint
 
@@ -53,7 +55,8 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.session = requests_cache.CachedSession(os.path.join(yfcu.GetUserCacheDirpath(),'yfinance.cache'))
         self.session.headers['User-agent'] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:77.0) Gecko/20100101 Firefox/77.0"
 
-        self.usa_tkr = "INTC"
+        # self.usa_tkr = "INTC"
+        self.usa_tkr = "GME" # Stock split recently
         self.usa_market = "us_market"
         self.usa_exchange = "NMS"
         self.usa_market_tz = ZoneInfo('US/Eastern')
@@ -74,6 +77,42 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.tempCacheDir.cleanup()
         self.session.close()
 
+    def verify_df(self, df, answer, rtol=None):
+        if df.shape[0] != answer.shape[0]:
+            raise Exception("Different #rows: df={}, answer={}".format(df.shape[0], answer.shape[0]))
+
+        dcs = ["Open","High","Low","Close","Volume","Dividends","Stock Splits"]
+        for dc in dcs:
+            if not (dc in df.columns and dc in answer.columns):
+                continue
+            if rtol is None:
+                f = (df[dc].values == answer[dc].values)
+            else:
+                f = np.isclose(df[dc].values, answer[dc].values, rtol=rtol)
+            try:
+                self.assertTrue(f.all())
+            except:
+                f = ~f
+                if sum(f) < 20:
+                    print("{}/{} differences in column {}:".format(sum(f), df.shape[0], dc))
+                    print("- answer:")
+                    print(answer[f][[dc]])
+                    print("- result:")
+                    print(df[f][[dc]])
+                else:
+                    print("{}/{} diffs in column {}".format(sum(f), df.shape[0], dc))
+
+                last_diff_idx = np.where(f)[0][-1]
+                x = df[dc][last_diff_idx]
+                y = answer[dc][last_diff_idx]
+                last_diff = x - y
+                print("- last_diff: {} - {} = {}".format(x, y, last_diff))
+                print("- answer:")
+                print(answer.iloc[last_diff_idx])
+                print("- result:")
+                print(df.iloc[last_diff_idx])
+                raise
+
 
     def test_history_basics1_usa(self):
         # A fetch of prices, then another fetch of same prices, should return identical
@@ -308,28 +347,31 @@ class Test_Yfc_Interface(unittest.TestCase):
         end_day_str = "2022-06-25"
 
         for aa in [False,True]:
-            for ba in [False,True]:
-                if aa and ba:
+            df_yf = dat_yf.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=False)
+            df_yfc = self.usa_dat.history(start=start_day_str, end=end_day_str, adjust_divs=aa)
+
+            for c in yfcd.yf_data_cols:
+                if not c in df_yf.columns:
                     continue
-
-                df_yf = dat_yf.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=ba)
-                df_yfc = self.usa_dat.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=ba)
-
-                # df_yf = dat_yf.history(start=start_dt, end=end_dt, auto_adjust=aa, back_adjust=ba)
-                # df_yfc = self.usa_dat.history(start=start_dt, end=end_dt, auto_adjust=aa, back_adjust=ba)
-
-                for c in yfcd.yf_data_cols:
-                    if not c in df_yf.columns:
-                        continue
-                    try:
-                        self.assertTrue(df_yf[c].equals(df_yfc[c]))
-                    except:
-                        print("df_yf:")
-                        print(df_yf[c])
-                        print("df_yfc:")
-                        print(df_yfc[[c,"FetchDate"]])
-                        print("aa={}, ba={}, c={}".format(aa, ba, c))
-                        raise
+                elif c == "Adj Close" and not c in df_yfc.columns:
+                    continue
+                try:
+                    if aa:
+                        self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=1e-10).all())
+                    else:
+                        self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
+                except:
+                    f = ~np.equal(df_yf[c].values, df_yfc[c].values)
+                    print("df_yf:")
+                    print(df_yf)
+                    print("df_yfc:")
+                    print(df_yfc)
+                    print("aa={}, c={}".format(aa, c))
+                    last_dt = df_yfc.index[f][-1]
+                    v1 = df_yfc.loc[last_dt][c]
+                    v2 =  df_yf.loc[last_dt][c]
+                    print("Last diff: {}: {} - {} = {}".format(last_dt, v1, v2, v1-v2))
+                    raise
 
 
     def test_matches_yf_daily_nze(self):
@@ -339,25 +381,26 @@ class Test_Yfc_Interface(unittest.TestCase):
         end_day_str = "2022-06-18"
 
         for aa in [False,True]:
-            for ba in [False,True]:
-                if aa and ba:
+            df_yf = dat_yf.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=False)
+            df_yfc = self.nze_dat.history(start=start_day_str, end=end_day_str, adjust_divs=aa)
+
+            for c in yfcd.yf_data_cols:
+                if not c in df_yf.columns:
                     continue
-
-                df_yf = dat_yf.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=ba)
-                df_yfc = self.nze_dat.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=ba)
-
-                for c in yfcd.yf_data_cols:
-                    if not c in df_yf.columns:
-                        continue
-                    try:
-                        self.assertTrue(df_yf[c].equals(df_yfc[c]))
-                    except:
-                        print("df_yf:")
-                        print(df_yf)
-                        print("df_yfc:")
-                        print(df_yfc)
-                        print("aa={}, ba={}, c={}".format(aa, ba, c))
-                        raise
+                elif c == "Adj Close" and not c in df_yfc.columns:
+                    continue
+                try:
+                    if aa:
+                        self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=1e-10).all())
+                    else:
+                        self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
+                except:
+                    print("df_yf:")
+                    print(df_yf)
+                    print("df_yfc:")
+                    print(df_yfc)
+                    print("aa={}, c={}".format(aa, c))
+                    raise
 
 
     def test_history_final(self):
@@ -412,9 +455,10 @@ class Test_Yfc_Interface(unittest.TestCase):
 
 
     def test_periods(self):
+        #
         tkrs = ["MEL.NZ", "IMP.JO", "INTC"]
-        # Add ticker with recent listing
-        tkrs.append("HLTH")
+        tkrs.append("HLTH") # Listed recently
+        tkrs.append("GME") # Stock split recently
         periods = [p for p in yfcd.Period]
         #
         for tkr in tkrs:
@@ -430,15 +474,10 @@ class Test_Yfc_Interface(unittest.TestCase):
                 df_yf = df_yf[np.isin(df_yf.index.date, sched["market_open"].dt.date)]
                 df_yf_backup = df_yf.copy()
 
-                df_yfc = dat_yfc.history(period=p, auto_adjust=False)
+                df_yfc = dat_yfc.history(period=p, adjust_divs=False)
 
                 ## How Yahoo maps period -> start_date is mysterious, so need to account for my different mapping:
-                d0 = df_yf.index.min()
-                d1 = df_yfc.index.min()
-                if d1>d0:
-                    td = d1-d0
-                else:
-                    td = d0-d1
+                td = abs(df_yf.index.min() - df_yfc.index.min())
                 if td < timedelta(days=28):
                     start_ts = max(df_yf.index.min(), df_yfc.index.min())
                     df_yf = df_yf[df_yf.index >= start_ts]
@@ -451,8 +490,6 @@ class Test_Yfc_Interface(unittest.TestCase):
                 except:
                     print("df_yf: {}".format(df_yf.shape))
                     print(df_yf)
-                    # print("df_yf 0-volume:")
-                    # print(df_yf[df_yf["Volume"]==0])
                     print("df_yfc: {}".format(df_yfc.shape))
                     print(df_yfc)
                     print("Different shapes")
@@ -460,26 +497,26 @@ class Test_Yfc_Interface(unittest.TestCase):
                 try:
                     self.assertTrue(df_yf.index.equals(df_yfc.index))
                 except:
-                        print("df_yf:")
-                        print(df_yf)
-                        print("df_yfc:")
-                        print(df_yfc)
-                        print("Index different")
-                        raise
+                    print("df_yf:")
+                    print(df_yf)
+                    print("df_yfc:")
+                    print(df_yfc)
+                    print("Index different")
+                    raise
                 f = df_yfc["Final?"].values
                 for c in yfcd.yf_data_cols:
-                    tol = 0.0
-                    if c == "Adj Close":
-                        tol = 1e-5
+                    if not c in df_yf.columns:
+                        continue
+                    elif c == "Adj Close" and not c in df_yfc.columns:
+                        continue
                     try:
                         if not f[-1]:
                             ## Ignore last row because data is live, can change between YF and YFC calls
-                            self.assertTrue(np.isclose(df_yf.loc[f,c].values, df_yfc.loc[f,c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf.loc[f,c].values, df_yfc.loc[f,c].values).all())
                         else:
-                            self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
                     except:
-                        # f = (df_yf[c]-df_yfc[c]).abs() > tol
-                        f = ~np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol)
+                        f = ~np.equal(df_yf[c].values, df_yfc[c].values)
                         print("Diff dates:")
                         print(df_yf.index[f])
                         print("Difference in column {}".format(c))
@@ -491,7 +528,7 @@ class Test_Yfc_Interface(unittest.TestCase):
 
                 # Fetch from cache should match
                 df_yf = df_yf_backup.copy()
-                df_yfc = dat_yfc.history(period=p, auto_adjust=False)
+                df_yfc = dat_yfc.history(period=p, adjust_divs=False)
                 start_ts = max(df_yf.index.min(), df_yfc.index.min())
                 df_yf = df_yf[df_yf.index >= start_ts]
                 df_yfc = df_yfc[df_yfc.index >= start_ts]
@@ -515,19 +552,18 @@ class Test_Yfc_Interface(unittest.TestCase):
                         raise
                 f = df_yfc["Final?"].values
                 for c in yfcd.yf_data_cols:
-                    tol = 0.0
-                    if c == "Adj Close":
-                        ## YF/Yahoo called at different times returns slightly different 'Adj Close', so need to tolerate negligible error
-                        tol = 1e-5
+                    if not c in df_yf.columns:
+                        continue
+                    elif c == "Adj Close" and not c in df_yfc.columns:
+                        continue
                     try:
                         if not f[-1]:
                             ## Ignore last row because data is live, can change between YF and YFC calls
-                            self.assertTrue(np.isclose(df_yf.loc[f,c].values, df_yfc.loc[f,c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf.loc[f,c].values, df_yfc.loc[f,c].values).all())
                         else:
-                            self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
                     except:
-                        # f = (df_yf[c]-df_yfc[c]).abs() > tol
-                        f = ~np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol)
+                        f = ~np.equal(df_yf[c].values, df_yfc[c].values)
                         print("Diff dates:")
                         print(df_yf.index[f])
                         print("Difference in column {}".format(c))
@@ -539,8 +575,8 @@ class Test_Yfc_Interface(unittest.TestCase):
 
     def test_periods_with_persistent_caching(self):
         tkrs = ["MEL.NZ", "IMP.JO", "INTC"]
-        # Add ticker with recent listing
-        tkrs.append("HLTH")
+        tkrs.append("HLTH") # Listed recently
+        tkrs.append("GME") # Stock split recently
         periods = [p for p in yfcd.Period]
         #
         for tkr in tkrs:
@@ -552,15 +588,10 @@ class Test_Yfc_Interface(unittest.TestCase):
                 sched = yfct.GetExchangeSchedule(dat_yfc.info["exchange"], df_yf.index[0].date(), df_yf.index[-1].date()+timedelta(days=1))
                 df_yf = df_yf[np.isin(df_yf.index.date, sched["market_open"].dt.date)]
 
-                df_yfc = dat_yfc.history(period=p, auto_adjust=False)
+                df_yfc = dat_yfc.history(period=p, adjust_divs=False)
 
                 ## How Yahoo maps period -> start_date is mysterious, so need to account for my different mapping:
-                d0 = df_yf.index.min()
-                d1 = df_yfc.index.min()
-                if d1>d0:
-                    td = d1-d0
-                else:
-                    td = d0-d1
+                td = abs(df_yf.index.min() - df_yfc.index.min())
                 if td < timedelta(days=28):
                     start_ts = max(df_yf.index.min(), df_yfc.index.min())
                     df_yf = df_yf[df_yf.index >= start_ts]
@@ -588,30 +619,29 @@ class Test_Yfc_Interface(unittest.TestCase):
                         raise
                 f = df_yfc["Final?"].values
                 for c in yfcd.yf_data_cols:
-                    tol = 0.0
-                    if c == "Adj Close":
-                        ## YF/Yahoo called at different times returns slightly different 'Adj Close', so need to tolerate negligible error
-                        tol = 1e-5
+                    if not c in df_yf.columns:
+                        continue
+                    elif c == "Adj Close" and not c in df_yfc.columns:
+                        continue
                     try:
                         if not f[-1]:
                             ## Ignore last row because data is live, can change between YF and YFC calls
-                            self.assertTrue(np.isclose(df_yf.loc[f,c].values, df_yfc.loc[f,c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf.loc[f,c].values, df_yfc.loc[f,c].values).all())
                         else:
-                            self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
                     except:
-                        # f = (df_yf[c]-df_yfc[c]).abs() > tol
-                        f = ~np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol)
+                        f = ~np.equal(df_yf[c].values, df_yfc[c].values)
                         print("Diff dates:")
                         print(df_yf.index[f])
                         print("Difference in column {}".format(c))
                         last_dt = df_yfc.index[f][-1]
                         v1 = df_yfc.loc[last_dt][c]
                         v2 =  df_yf.loc[last_dt][c]
-                        print("Last diff: {}: {} - {} = {}".format(last_dt, v1, v2, v1-v2))
+                        print("Last diff: {}: YFC={} - YF={} = {}".format(last_dt, v1, v2, v1-v2))
                         raise
 
                 # Fetch from cache should match
-                df_yfc = dat_yfc.history(period=p, auto_adjust=False)
+                df_yfc = dat_yfc.history(period=p, adjust_divs=False)
                 df_yfc = df_yfc[df_yfc.index >= start_ts]
                 try:
                     self.assertEqual(df_yf.shape[0], df_yfc.shape[0])
@@ -635,19 +665,18 @@ class Test_Yfc_Interface(unittest.TestCase):
                         raise
                 f = df_yfc["Final?"].values
                 for c in yfcd.yf_data_cols:
-                    tol = 0.0
-                    if c == "Adj Close":
-                        ## YF/Yahoo called at different times returns slightly different 'Adj Close', so need to tolerate negligible error
-                        tol = 1e-5
+                    if not c in df_yf.columns:
+                        continue
+                    elif c == "Adj Close" and not c in df_yfc.columns:
+                        continue
                     try:
                         if not f[-1]:
                             ## Ignore last row because data is live, can change between YF and YFC calls
-                            self.assertTrue(np.isclose(df_yf.loc[f,c].values, df_yfc.loc[f,c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf.loc[f,c].values, df_yfc.loc[f,c].values).all())
                         else:
-                            self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol).all())
+                            self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
                     except:
-                        # f = (df_yf[c]-df_yfc[c]).abs() > tol
-                        f = ~np.isclose(df_yf[c].values, df_yfc[c].values, rtol=tol)
+                        f = ~np.equal(df_yf[c].values, df_yfc[c].values)
                         print("Diff dates:")
                         print(df_yf.index[f])
                         print("Difference in column {}".format(c))
@@ -680,8 +709,6 @@ class Test_Yfc_Interface(unittest.TestCase):
                 tz = ZoneInfo(dat.info["exchangeTimezoneName"])
                 while d < end_dt.date():
                     if yfct.ExchangeOpenOnDay(exchange, d):
-                        # dt = datetime.combine(d, time(0))
-                        # dt = tz.localize(dt)
                         dt = datetime.combine(d, time(0), tz)
                         expected_interval_dates.append(dt)
                     d += timedelta(days=1)
@@ -727,13 +754,27 @@ class Test_Yfc_Interface(unittest.TestCase):
                 sleep(10)
                 n = df1.shape[0]
                 df3 = dat.history(interval="1d", start=start_d, end=end_d, max_age=timedelta(seconds=1))
+                try:
+                    self.assertEqual(len(expected_interval_dates), df3.shape[0])
+                except:
+                    print("Different shapes")
+                    print("df1:")
+                    print(df1)
+                    print("df3:")
+                    print(df3)
+                    raise
                 try:
                     self.assertTrue(np.array_equal(expected_interval_dates, df3.index))
                 except:
                     print("expected_interval_dates:")
                     pprint(expected_interval_dates)
                     print("df3:")
-                    print(df3)
+                    print(df3.index)
+                    edt0 = expected_interval_dates[0]
+                    adt0 = df3.index[0]
+                    print("expected dt0 = {} (tz={})".format(edt0, edt0.tzinfo))
+                    print("actual dt0 = {} (tz={})".format(adt0, adt0.tzinfo))
+                    print(expected_interval_dates == df3.index)
                     raise
                 try:
                     self.assertTrue(df1.iloc[0:n-1].equals(df3.iloc[0:n-1]))
@@ -751,15 +792,17 @@ class Test_Yfc_Interface(unittest.TestCase):
                 for c in yfcd.yf_data_cols:
                     if not c in df_yf.columns:
                         continue
+                    elif c == "Adj Close" and not c in df_yfc.columns:
+                        continue
                     try:
-                        self.assertTrue(df_yf[c].equals(df1[c]))
+                        self.assertTrue(np.array_equal(df_yf[c].values, df1[c].values))
                     except:
                         print("")
                         print("df_yf:")
-                        print(df_yf)
+                        print(df_yf[[c]])
                         print("")
                         print("df_yfc:")
-                        print(df1)
+                        print(df1[[c]])
                         print("")
                         print("Difference in column {}".format(c))
                         raise
@@ -770,11 +813,12 @@ class Test_Yfc_Interface(unittest.TestCase):
         interval = yfcd.Interval.Hours1
         #
         dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
-        d_now = dt_now.date()
         for tkr in tkr_candidates:
             dat = yfc.Ticker(tkr, session=None)
             exchange = dat.info["exchange"]
-            yfct.SetExchangeTzName(exchange, dat.info["exchangeTimezoneName"])
+            tz = dat.info["exchangeTimezoneName"]
+            yfct.SetExchangeTzName(exchange, tz)
+            d_now = dt_now.astimezone(ZoneInfo(tz)).date()
             if yfct.ExchangeOpenOnDay(exchange, d_now):
                 sched = yfct.GetExchangeSchedule(exchange, d_now, d_now+timedelta(days=1))
                 if (not sched is None) and dt_now > sched["market_close"][0]:
@@ -782,6 +826,9 @@ class Test_Yfc_Interface(unittest.TestCase):
                     start_dt = sched["market_open"][0].to_pydatetime()
                     end_dt = sched["market_close"][0].to_pydatetime()
 
+                    # Ensure Yahoo returns correct volume for first interval:
+                    start_dt -= timedelta(minutes=20)
+
                     dt = sched["market_open"][0]
                     expected_interval_starts = []
                     while dt < sched["market_close"][0]:
@@ -802,21 +849,26 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1h", start=start_dt, end=end_dt)
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        try:
-                            self.assertTrue(df_yf[c].equals(df1[c]))
-                        except:
-                            print("")
-                            print("df_yf:")
-                            print(df_yf)
-                            print("")
-                            print("df_yfc:")
-                            print(df1)
-                            print("")
-                            print("{}: Difference in column {}".format(tkr, c))
-                            raise
+                    # Note: Yahoo doesn't dividend-adjust hourly
+                    df1 = dat.history(start=start_dt, end=end_dt, interval="1h", adjust_divs=False)
+                    df_yf = dat_yf.history(start=start_dt, interval="1h")
+                    # Discard 0-volume data at market close
+                    td_1d = timedelta(days=1)
+                    sched = yfct.GetExchangeSchedule(dat.info["exchange"], df_yf.index.date.min(), df_yf.index.date.max()+td_1d)
+                    sched["_date"] = sched.index.date
+                    df_yf["_date"] = df_yf.index.date
+                    answer2 = df_yf.merge(sched, on="_date", how="left", validate="many_to_one")
+                    answer2.index = df_yf.index ; df_yf = answer2
+                    f_drop = (df_yf["Volume"]==0).values & ((df_yf.index<df_yf["market_open"]) | (df_yf.index>=df_yf["market_close"]))
+                    df_yf = df_yf[~f_drop].drop("_date",axis=1)
+                    # YF hourly volume is not split-adjusted, so adjust:
+                    ss = df_yf["Stock Splits"].copy()
+                    ss[ss==0.0] = 1.0
+                    ss_rcp = 1.0/ss
+                    csf = ss_rcp.sort_index(ascending=False).cumprod().sort_index(ascending=True).shift(-1, fill_value=1.0)
+                    df_yf["Volume"] /= csf
+                    df_yf = df_yf[df_yf.index<end_dt]
+                    self.verify_df(df1, df_yf, 1e-10)
 
     def test_history_live_1d_evening(self):
         # Fetch during evening after active session
@@ -858,21 +910,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1d", start=start_dt.date(), end=end_dt.date())
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        try:
-                            self.assertTrue(df_yf[c].equals(df1[c]))
-                        except:
-                            print("")
-                            print("df_yf:")
-                            print(df_yf)
-                            print("")
-                            print("df_yfc:")
-                            print(df1)
-                            print("")
-                            print("Difference in column {}".format(c))
-                            raise
+                    self.verify_df(df1, df_yf, 1e-10)
 
     def test_history_live_1w_evening(self):
         # Fetch during evening after active session
@@ -920,21 +958,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1wk", start=start_dt.date(), end=end_dt.date())
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        try:
-                            self.assertTrue(df_yf[c].equals(df1[c]))
-                        except:
-                            print("Difference in column {}".format(c))
-                            print("")
-                            print("df_yf:")
-                            print(df_yf)
-                            print("")
-                            print("df_yfc:")
-                            print(df1)
-                            print("")
-                            raise
+                    self.verify_df(df1, df_yf)
 
     def test_history_jse_is_weird(self):
         # JSE market closes at 5pm local time, but if request 1h data 
@@ -942,21 +966,25 @@ class Test_Yfc_Interface(unittest.TestCase):
         # This mysterious row disappears tomorrow.
         interval = "1h"
         tkr = "IMP.JO"
-
-        dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
-        d_now = dt_now.date()
         dat = yfc.Ticker(tkr, session=None)
         exchange = dat.info["exchange"]
-        yfct.SetExchangeTzName(exchange, dat.info["exchangeTimezoneName"])
+        tz_name = dat.info["exchangeTimezoneName"]
+        tz = ZoneInfo(tz_name)
+        yfct.SetExchangeTzName(exchange, tz_name)
+
+        dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
+        # d_now = dt_now.date()
+        d_now = dt_now.astimezone(tz).date()
+        td_1d = timedelta(days=1)
         if yfct.ExchangeOpenOnDay(exchange, d_now):
-            sched = yfct.GetExchangeSchedule(exchange, d_now, d_now+timedelta(days=1))
-            if (not sched is None) and dt_now > sched["market_close"][0]:
-                start_dt = dt_now - timedelta(days=7)
-                end_dt = dt_now
-                tz = ZoneInfo(dat.info["exchangeTimezoneName"])
+            sched = yfct.GetExchangeSchedule(exchange, d_now-5*td_1d, d_now+td_1d)
+            if not sched is None:
+                for i in range(sched.shape[0]-1, -1, -1):
+                    if dt_now >= sched["market_close"].iloc[i]:
+                        break
+                d = sched["market_close"].iloc[i].date()
+                df = dat.history(start=d, end=d+td_1d, interval="1h")
 
-                ## Test is simple. Fetch should complete, and no 5pm in table
-                df = dat.history(start=d_now, interval="1h")
                 try:
                     self.assertEqual(df.shape[0], 8)
                     self.assertEqual(df.index[-1].time(), time(16))
@@ -1012,12 +1040,12 @@ class Test_Yfc_Interface(unittest.TestCase):
             self.assertTrue(np.isnan(df["Close"][idx]))
 
 if __name__ == '__main__':
-    # Run tests sequentially:
+    unittest.main()
+
+    # # Run tests sequentially:
     # import inspect
     # test_src = inspect.getsource(Test_Yfc_Interface)
     # unittest.TestLoader.sortTestMethodsUsing = lambda _, x, y: (
     #     test_src.index(f"def {x}") - test_src.index(f"def {y}")
     # )
-    #
-    unittest.main()
     # unittest.main(verbosity=2)
